package arrietty.test;


import java.net.UnknownHostException;
import java.util.Iterator;

import org.apache.log4j.Logger;
import org.jongo.Jongo;
import org.jongo.MongoCollection;
import org.jongo.MongoCursor;
import org.jongo.ResultHandler;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;

import com.mongodb.DB;
import com.mongodb.DBObject;
import com.mongodb.MongoClient;
import com.mongodb.MongoClientOptions;
import com.mongodb.ServerAddress;
import com.mongodb.WriteConcern;
import com.mongodb.WriteResult;
import com.mongodb.util.JSON;
import com.mongodb.util.JSONSerializers;

@Controller
@RequestMapping(value = "/")
public class MongoPerfTest {
	private static final Logger log = Logger.getLogger(MongoPerfTest.class);
	private MongoClient mongo;
/*
 * DB INDEX
 *

       db.testPerf.ensureIndex({
       'RegistryObjectList.RegistryPackage.ExternalIdentifier.value': 1,
       'RegistryObjectList.RegistryPackage.ExternalIdentifier.identificationScheme': 1}, {name: 'RegistryPackageExtidIndex'});

       db.testPerf.ensureIndex({'RegistryObjectList.Association.targetObject': 1}, {sparse: true, name : 'AssociationTargetIndex'});

       db.testPerf.ensureIndex({'RegistryObjectList.Association.sourceObject': 1}, {sparse: true, name : 'AssociationSourceIndex'});

       db.testPerf.ensureIndex({'RegistryObjectList.Classification.classificationNode': 1, 'RegistryObjectList.Classification.classifiedObject': 1}, {sparse: true, name : 'ClassificationIndex'});

       db.testPerf.ensureIndex({'uuids': 1}, {sparse: true, name : 'UUIDSIndex'});

        db.testPerf.ensureIndex({
       'RegistryObjectList.ExtrinsicObject.ExternalIdentifier.value' : 1,
       'RegistryObjectList.ExtrinsicObject.ExternalIdentifier.identificationScheme': 1}, { sparse: true, name: 'ExtrinsicObjectExtIdIndex'});

        db.testPerf.ensureIndex({'RegistryObjectList.insertTime': 1}, { name: 'insertDtIndex'});
        db.testPerf.ensureIndex({'pendingTransactions': 1}, { sparse: true,  name: 'pendingTransactionsIndex'});
        db.testPerf.ensureIndex({'RegistryObjectList.patientId': 1,'referenceIdList.refIds':1}, {sparse: true, name : 'refIdsIndex'});
        db.testPerf.ensureIndex({'RegistryObjectList.Association.id': 1}, {sparse: true, name : 'AssociationIdIndex'});

        db.testPerf.ensureIndex({'RegistryObjectList.ExtrinsicObject.lid' : 1},{sparse: true, 'unique': false, name : 'ExtrinsicObjectLidIndex'});
        db.testPerf.ensureIndex({'RegistryObjectList.RegistryPackage.lid' : 1},{'unique': false, name: 'RegistryPackageLidIndex'});
 */

	public MongoPerfTest() throws UnknownHostException {
		super();
		 com.mongodb.MongoClientOptions.Builder mongoOptionsBuilder = MongoClientOptions.builder();
	     mongoOptionsBuilder.writeConcern(WriteConcern.JOURNALED);
	     mongo = new MongoClient(new ServerAddress("localhost", 27017), mongoOptionsBuilder.build());
	}

	public class JSONResultHandler implements ResultHandler<String> {
		@Override
		public String map(DBObject result) {
			return JSONSerializers.getStrict().serialize(result);
		}

	}

	@RequestMapping(method = RequestMethod.POST, value = "/newDocument", headers = "content-type=application/json")
	public ResponseEntity<String> applicationUpsert(@RequestBody String json) {
		HttpHeaders responseHeaders = new HttpHeaders();
        responseHeaders.setContentType(MediaType.valueOf("application/json"));
		try {
			DB db = mongo.getDB("test");
			Jongo jongo = new Jongo(db);

			String query = "{'RegistryObjectList.ExtrinsicObject' : {$elemMatch : { 'ExternalIdentifier.value' : {$in: ['73387957557175793898']}, "
					+ "'Slot.name': 'hash', 'ExternalIdentifier.identificationScheme': 'urn:uuid:2e82c1f6-a085-4c72-9da3-8640a32e42ab'}}}";
			MongoCollection submission = jongo.getCollection("testPerf");
			Iterable<String> documentsAsJson = submission.find(query).map(new JSONResultHandler());
			Iterator<String> iter = documentsAsJson.iterator();
			String s;
			if (iter.hasNext()) {
				while (iter.hasNext()) {
					s = iter.next();
				}
			}
			DBObject jsonColl = (DBObject) JSON.parse(json);

			WriteResult res = submission.insert(jsonColl);
			return new ResponseEntity<String>("{\"response\": \"OK\"}",
                    responseHeaders,
                    HttpStatus.CREATED);

		} catch (Exception e) {
			log.error(e.getMessage(), e);
			return new ResponseEntity<String>("{\"response\": \"KO\", \"errorMessage\": \"" + e.getMessage() + "\"}",
                    responseHeaders,
                    HttpStatus.INTERNAL_SERVER_ERROR);
		}
	}

}
